@@ -54,6 +54,7 @@
 #include "request.h"
 #include "settings.h"
 #include "wallpaper.h"
+#include "dynamic-launcher.h"
 
 
 static GMainLoop *loop = NULL;
@@ -170,6 +171,12 @@ on_bus_acquired (GDBusConnection *connection,
       g_warning ("error: %s\n", error->message);
       g_clear_error (&error);
     }
+
+  if (!dynamic_launcher_init (connection, &error))
+    {
+      g_warning ("error: %s\n", error->message);
+      g_clear_error (&error);
+    }
 }
 
 static void
@@ -204,8 +211,18 @@ main (int argc, char *argv[])
 
   /* Avoid pointless and confusing recursion */
   g_unsetenv ("GTK_USE_PORTAL");
-  g_setenv ("ADW_DISABLE_PORTAL", "1", TRUE);
-  g_setenv ("GSK_RENDERER", "cairo", TRUE);
+
+  if (G_UNLIKELY (!g_setenv ("ADW_DISABLE_PORTAL", "1", TRUE)))
+    {
+      g_printerr ("Failed to set ADW_DISABLE_PORTAL: %s\n", g_strerror (errno));
+      return 1;
+    }
+
+  if (G_UNLIKELY (!g_setenv ("GSK_RENDERER", "cairo", TRUE)))
+    {
+      g_printerr ("Failed to set GSK_RENDERER: %s\n", g_strerror (errno));
+      return 1;
+    }
 
   gtk_init ();
 
